/**
 * 
 */
package com.axemble.education.connectors.navigation.common;

import org.w3c.dom.Element;

import com.axemble.vdoc.sdk.connectors.BaseServiceConnector;
import com.axemble.vdoc.sdk.exceptions.SDKException;
import com.axemble.vdoc.sdk.interfaces.IConnectorExecutionStatus;
import com.axemble.vdoc.sdk.interfaces.IContext;
import com.axemble.vdoc.sdk.interfaces.IResourceController;
import com.axemble.vdoc.sdk.interfaces.IServiceDefinition.IEvent;
import com.axemble.vdp.ui.framework.foundation.Navigator;

/**
 * @author vlygeros
 *
 */
public abstract class CommonNavigationServiceConnector extends BaseServiceConnector
{
	private static final long serialVersionUID = 1L;

	protected static final String OUTPUT_RESULT = "result";
	protected static final String OUTPUT_ERROR_NUMBER = "errorNumber";
	protected static final String OUTPUT_ERROR_MESSAGE = "errorMessage";

	/* (non-Javadoc)
	 * @see com.axemble.vdoc.sdk.connectors.BaseServiceConnector#execute(com.axemble.vdoc.sdk.interfaces.IContext, int, java.lang.String, org.w3c.dom.Element)
	 */
	@Override
	public int execute( IContext context, int eventType, String eventName, Element customElement )
	{
		if ( eventType == IEvent.BACK_END )
		{
			this.getServiceOutputs().addOutput( OUTPUT_RESULT, Boolean.FALSE );
			this.getServiceOutputs().addOutput( OUTPUT_ERROR_MESSAGE, "Only frontend event are supported!" );
			this.getServiceOutputs().addOutput( OUTPUT_ERROR_NUMBER, -900 );

			return IConnectorExecutionStatus.STOP;
		}

		try
		{
			IResourceController resourceController = getWorkflowModule().getResourceController( getResource() );
			execute( resourceController );
		}
		catch( Exception e )
		{
			String errorMessage = getErrorMessage();
			buildError( eventType, errorMessage, e );

			this.getServiceOutputs().addOutput( OUTPUT_RESULT, Boolean.FALSE );
			this.getServiceOutputs().addOutput( OUTPUT_ERROR_MESSAGE, errorMessage );
			this.getServiceOutputs().addOutput( OUTPUT_ERROR_NUMBER, -900 );

			return IConnectorExecutionStatus.STOP;
		}

		this.getServiceOutputs().addOutput( OUTPUT_RESULT, Boolean.TRUE );

		return IConnectorExecutionStatus.STOP_ASYNCHRONOUS;
	}

	abstract protected void execute( IResourceController resourceController );

	protected void buildError( int eventType, String errorMessage, Exception e )
	{
		if ( IEvent.FRONT_END == eventType )
			Navigator.getNavigator().getRootNavigator().showAlertBox( errorMessage );
		else if ( eventType == IEvent.BACK_END )
		{
			if ( e != null )
				throw new SDKException( errorMessage, e );
			else throw new SDKException( errorMessage );
		}
	}
	protected String getErrorMessage()
	{
		return getStaticString( "toto", this.getServiceDefinition().getLabel() );
	}
}
